@@ -227,7 +227,8 @@ class BasePiece(metaclass=abc.ABCMeta):
         piece_input_model: pydantic.BaseModel,
         piece_output_model: pydantic.BaseModel,
         piece_secrets_model: Optional[pydantic.BaseModel] = None,
-        airflow_context: Optional[dict] = None
+        airflow_context: Optional[dict] = None,
+        workspace_id: Optional[int] = None
     ):
         """
         _summary_
@@ -281,6 +282,7 @@ class BasePiece(metaclass=abc.ABCMeta):
         call_piece_func_dict = {"input_data": input_model_obj}
         if piece_secrets_model:
             call_piece_func_dict['secrets_data'] = secrets_model_obj
+        call_piece_func_dict['workspace_id'] = workspace_id
         output_obj = self.piece_function(**call_piece_func_dict)
 
         # Validate output data
