@@ -1,3 +1,5 @@
+from starflow.base_piece import BaseBranchOutputModel
+from starflow.scripts.load_piece import load_piece_branch_models_from_path
 import tomli
 import tomli_w
 import json
@@ -268,13 +270,26 @@ def create_compiled_pieces_metadata(source_url: str | None = None) -> None:
                 metadata['name'] = piece_name
 
             # Add input and output schemas
-            input_model_class, output_model_class, secrets_model_class = load_piece_models_from_path(
-                pieces_folder_path=str(pieces_path),
-                piece_name=op_dir.name
-            )
-            metadata["input_schema"] = input_model_class.model_json_schema()
-            metadata["output_schema"] = output_model_class.model_json_schema()
-            metadata["secrets_schema"] = secrets_model_class.model_json_schema() if secrets_model_class else None
+
+            if metadata["branchPiece"] == True:
+
+                input_model_class = load_piece_branch_models_from_path(
+                    pieces_folder_path=str(pieces_path),
+                    piece_name=op_dir.name
+                )
+                metadata["input_schema"] = input_model_class.model_json_schema()
+                metadata["output_schema"] = BaseBranchOutputModel.model_json_schema()
+                metadata["secrets_schema"] = None
+
+            else:
+
+                input_model_class, output_model_class, secrets_model_class = load_piece_models_from_path(
+                    pieces_folder_path=str(pieces_path),
+                    piece_name=op_dir.name
+                )
+                metadata["input_schema"] = input_model_class.model_json_schema()
+                metadata["output_schema"] = output_model_class.model_json_schema()
+                metadata["secrets_schema"] = secrets_model_class.model_json_schema() if secrets_model_class else None
 
             # Add source code url
             metadata["source_url"] = None
