@@ -15,7 +15,7 @@ from starflow.schemas import WorkflowSharedStorage, ContainerResourcesModel
 from starflow.storage.s3 import S3StorageRepository
 from starflow.logger import get_configured_logger
 from airflow.exceptions import AirflowException
-from airflow.kubernetes.pod_generator import PodDefaults
+from airflow.providers.cncf.kubernetes.utils.xcom_sidecar import PodDefaults
 import json
 import os
 from airflow.operators.branch import BaseBranchOperator
@@ -30,14 +30,26 @@ class starflowKubernetesBranchPodOperator(KubernetesPodOperator, BaseBranchOpera
         repository_url: str,
         repository_version: str,
         workspace_id: int,
-        task_id_main_branch: str,
-        task_id_else_branch: str,
+        task_id_main_branch: str | Iterable[str],
+        task_id_else_branch: str | Iterable[str],
         username: Optional[str] = None,
         piece_input_kwargs: Optional[Dict] = None,
         workflow_shared_storage: WorkflowSharedStorage = None,
         container_resources: Optional[Dict] = None,
         **k8s_operator_kwargs
     ):
+        import logging
+        logger = logging.getLogger(__name__)
+        
+        logger.info("=" * 80)
+        logger.info(f"[StarflowK8sBranchOperator] INIT METHOD CALLED - task_id: {task_id}")
+        logger.info(f"[StarflowK8sBranchOperator] dag_id: {dag_id}")
+        logger.info(f"[StarflowK8sBranchOperator] piece_name: {piece_name}")
+        logger.info(f"[StarflowK8sBranchOperator] repository_url: {repository_url}")
+        logger.info(f"[StarflowK8sBranchOperator] repository_version: {repository_version}")
+        logger.info(f"[StarflowK8sBranchOperator] workspace_id: {workspace_id}")
+        logger.info("=" * 80)
+        
         self.logger = get_configured_logger("starflowKubernetesBranchPodOperator")
         self.task_id = task_id
         self.piece_name = piece_name
@@ -55,6 +67,7 @@ class starflowKubernetesBranchPodOperator(KubernetesPodOperator, BaseBranchOpera
         # Environment variables
         pod_env_vars = {
             "STARFLOW_PIECE": piece_name,
+            "WORKSPACE_ID": str(workspace_id),
             "STARFLOW_INSTANTIATE_PIECE_KWARGS": str({
                 "deploy_mode": deploy_mode,
                 "task_id": task_id,
@@ -213,9 +226,27 @@ class starflowKubernetesBranchPodOperator(KubernetesPodOperator, BaseBranchOpera
         self.task_id_replaced = self.task_id.lower().replace("_", "-") # doing this because airflow doesn't allow underscores and upper case in mount names and max len is 63
         self.shared_storage_base_mount_path = '/home/shared_storage'
 
+        # Fix pod name to comply with Kubernetes naming rules (no underscores)
+        if pod.metadata.name:
+            # Replace underscores with hyphens and ensure it's lowercase
+            pod.metadata.name = pod.metadata.name.lower().replace("_", "-")
+            # Ensure it starts and ends with alphanumeric characters
+            pod.metadata.name = pod.metadata.name.strip("-")
+            if not pod.metadata.name[0].isalnum():
+                pod.metadata.name = "pod-" + pod.metadata.name
+            if not pod.metadata.name[-1].isalnum():
+                pod.metadata.name = pod.metadata.name + "-pod"
+
         pod.metadata.namespace = "airflow"
         pod.spec.image_pull_secrets = [k8s.V1LocalObjectReference('ghcr-login-secret')]
 
+        # Override xcom sidecar image to use Harbor registry instead of quay.io
+        if pod.spec.containers:
+            for container in pod.spec.containers:
+                if container.name == "airflow-xcom-sidecar":
+                    container.image = "harbor.prochain.name/public/alpine:latest"
+                    self.logger.info(f"[StarflowK8sBranchOperator] Updated xcom sidecar image to: {container.image}")
+
         # if not self.workflow_shared_storage or self.workflow_shared_storage.mode.name == 'none':
         #     return pod
         # if self.workflow_shared_storage.source.name in ["aws_s3", "gcs"]:
@@ -225,7 +256,6 @@ class starflowKubernetesBranchPodOperator(KubernetesPodOperator, BaseBranchOpera
 
         return pod
 
-
     def add_local_shared_storage_volumes(self, pod: k8s.V1Pod) -> k8s.V1Pod:
         """Adds local shared storage volumes to the pod."""
         pod_cp = copy.deepcopy(pod)
@@ -396,6 +426,11 @@ class starflowKubernetesBranchPodOperator(KubernetesPodOperator, BaseBranchOpera
         piece_repository_data = self.starflow_client.get_piece_repositories_from_workspace_id(
             params=params
         ).json()
+        
+        # Check if any piece repositories were found
+        if not piece_repository_data.get("data") or len(piece_repository_data["data"]) == 0:
+            return {}
+        
         secrets_response = self.starflow_client.get_piece_secrets(
             piece_repository_id=piece_repository_data["data"][0]["id"],
             piece_name=piece_name
@@ -466,8 +501,26 @@ class starflowKubernetesBranchPodOperator(KubernetesPodOperator, BaseBranchOpera
         Runs at the begining of the execute method.
         Pass extra arguments and configuration as environment variables to the pod
         """
+        import logging
+        logger = logging.getLogger(__name__)
+        
         # Fetch upstream tasks ids and save them in an ENV var
-        upstream_task_ids = [t.task_id for t in self.get_direct_relatives(upstream=True)]
+        # Check if operator is properly assigned to a DAG
+        try:
+            # Try to access the dag property - this will raise RuntimeError if not assigned
+            dag = self.dag
+            if dag is None:
+                logger.warning("[StarflowK8sBranchOperator] Operator DAG is None, skipping upstream task IDs")
+                upstream_task_ids = []
+            else:
+                upstream_task_ids = [t.task_id for t in self.get_direct_relatives(upstream=True)]
+                logger.info(f"[StarflowK8sBranchOperator] Upstream task IDs: {upstream_task_ids}")
+        except RuntimeError as e:
+            if "has not been assigned to a DAG yet" in str(e):
+                logger.warning("[StarflowK8sBranchOperator] Operator not assigned to a DAG, skipping upstream task IDs")
+                upstream_task_ids = []
+            else:
+                raise
         self.env_vars.append({
             'name': 'AIRFLOW_UPSTREAM_TASKS_IDS',
             'value': str(upstream_task_ids),
@@ -517,19 +570,49 @@ class starflowKubernetesBranchPodOperator(KubernetesPodOperator, BaseBranchOpera
         """
         Code from here onward is executed by the Worker and not by the Scheduler.
         """
-        # TODO change url based on platform configuration
-        self.starflow_client = starflowBackendRestClient(base_url="http://starflow-sa-rest-service.prochain-starflow:8000/")
-        self._prepare_execute_environment(context=context)
-        remote_pod = None
+        import logging
+        logger = logging.getLogger(__name__)
+        
+        # Log at the very beginning to confirm execute method is called
+        logger.info("=" * 80)
+        logger.info(f"[StarflowK8sBranchOperator] EXECUTE METHOD CALLED - task_id: {self.task_id}")
+        logger.info(f"[StarflowK8sBranchOperator] Context type: {type(context)}")
+        logger.info(f"[StarflowK8sBranchOperator] Context keys: {list(context.keys()) if hasattr(context, 'keys') else 'No keys method'}")
+        logger.info("=" * 80)
+        
         try:
+            logger.info(f"[StarflowK8sBranchOperator] Starting execution for task_id: {self.task_id}")
+            
+            # Initialize remote_pod early to avoid UnboundLocalError in finally block
+            remote_pod = None
+            
+            # TODO change url based on platform configuration
+            logger.info("[StarflowK8sBranchOperator] Initializing Starflow backend client...")
+            self.starflow_client = starflowBackendRestClient(base_url="http://starflow-rest-service.prochain-starflow:8000/")
+            logger.info("[StarflowK8sBranchOperator] Starflow backend client initialized successfully")
+            
+            logger.info("[StarflowK8sBranchOperator] Preparing execute environment...")
+            self._prepare_execute_environment(context=context)
+            logger.info("[StarflowK8sBranchOperator] Execute environment prepared successfully")
+            logger.info("[StarflowK8sBranchOperator] Building pod request object...")
             self.pod_request_obj = self.build_pod_request_obj(context)
+            logger.info("[StarflowK8sBranchOperator] Pod request object built successfully")
+            
+            logger.info("[StarflowK8sBranchOperator] Creating/getting pod...")
             self.pod = self.get_or_create_pod(  # must set `self.pod` for `on_kill`
                 pod_request_obj=self.pod_request_obj,
                 context=context,
             )
+            logger.info(f"[StarflowK8sBranchOperator] Pod created successfully: {self.pod.metadata.name}")
+            
             # get remote pod for use in cleanup methods
+            logger.info("[StarflowK8sBranchOperator] Finding remote pod...")
             remote_pod = self.find_pod(self.pod.metadata.namespace, context=context)
+            logger.info("[StarflowK8sBranchOperator] Remote pod found successfully")
+            
+            logger.info("[StarflowK8sBranchOperator] Awaiting pod start...")
             self.await_pod_start(pod=self.pod)
+            logger.info("[StarflowK8sBranchOperator] Pod started successfully")
 
             if self.get_logs:
                 self.pod_manager.fetch_container_logs(
