@@ -28,6 +28,7 @@ class Task(object):
         workflow_shared_storage: dict = None,
         container_resources: dict = None,
         username: Optional[str] = None,
+        trigger_rule="all_success",
         **kwargs
     ) -> None:
         # Task configuration and attributes
@@ -42,6 +43,7 @@ class Task(object):
         self.piece = piece
         self.piece_input_kwargs = piece_input_kwargs
         self.username_var = username
+        self.trigger_rule = trigger_rule
         if "execution_mode" not in self.piece:
             self.execution_mode = "docker"
         else:
@@ -64,10 +66,26 @@ class Task(object):
         self.container_resources = container_resources
 
         # Get deploy mode
-        self.deploy_mode = os.environ.get('STARFLOW_DEPLOY_MODE')
+        self.deploy_mode = os.environ.get('STARFLOW_DEPLOY_MODE', 'prod')
 
         # Set up task operator
         self._task_operator = self._set_operator()
+    
+    def _construct_image_url(self) -> str:
+        """Construct the Docker image URL from repository_url and piece name"""
+        # Extract the base image name from repository_url
+        # repository_url is like: ghcr.io/prochain-star-atlas/prochain-starflow-pieces:0.1.75-group0
+        # We need to construct: ghcr.io/prochain-star-atlas/prochain-starflow-pieces:0.1.75-group0
+        
+        # For now, use the repository_url as the base and append the piece name
+        # This matches the pattern we see in the logs
+        base_image = self.repository_url
+        if ':' in base_image:
+            # If it already has a tag, use it as is
+            return base_image
+        else:
+            # If no tag, append the version
+            return f"{base_image}:{self.repository_version}"
 
     def _set_operator(self) -> BaseOperator:
         """
@@ -94,8 +112,8 @@ class Task(object):
             container_resources=self.container_resources,
             # ----------------- Kubernetes -----------------
             namespace='airflow',
-            image=self.piece.get("source_image"),
-            image_pull_policy='Always',
+            image=self.piece.get("source_image") or self._construct_image_url(),
+            image_pull_policy='IfNotPresent',
             name=f"airflow-worker-pod-{self.task_id}",
             startup_timeout_seconds=600,
             annotations={"sidecar.istio.io/inject": "false"}, # TODO - remove this when istio is working with airflow k8s pod
@@ -118,7 +136,8 @@ class Task(object):
                         secret_name="airflow-worker-secrets"
                     )
                 )
-            ]
+            ],
+            trigger_rule=self.trigger_rule
         )
             
 
